/**
 * A library to interface with the Vixid VJX16-4 controller.
 *
 * (c) 2010
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @author		Julien Deswaef (aka. Juego) http://xuv.be
 * @modified	11/04/2010
 * @version		0.1.2
 */

package vixid.library;

import processing.core.*;
import themidibus.*; 

/**
 * The Vixid class provides a simple way to communicate with the Vixid video controller
 * within Processing sketches.
 * 
 * It uses the theMidiBus library for handling Midi communications (so you need 
 * to have that library in your sketchbook folder to make it work). This library is
 * mostly a set of functions that mimic what you can find on the Vixid controller.
 * 
 * @example Hello
 * 
 */

public class Vixid {

	// app is a reference to the parent sketch
	PApplet app;

	MidiBus midibus; 

	public Track[] track = new Track[4];

	int mixMode = 0; // 0 : Compositing mode, 1 : Battle 2*2 mode
	
	int[] output = new int[3];  // the 3 outputs
	int[] preview = new int[3]; // the 3 previews

	/*
	 * TODO: Understand how wipes work and how to set it up
	 */
	boolean wipes = false;

	int myVariable = 0;

	public final static String VERSION = "0.1.2";


	/**
	 * Create an instance of the Vixid object.
	 * Tell him where to get it's midi command from/to.
	 * All outputs are set to Master.
	 * 
	 * @example Hello
	 * @param theParent
	 * @param midiInputIndex
	 * 			The midi index of the midi input. See midiList() for reference
	 * @param midiOutputIndex
	 * 			The midi index of the midi output. See midiList() for reference
	 * 
	 */
	
	public Vixid(PApplet theParent, int midiInputIndex, int midiOutputIndex, boolean[] videoIn) {
		app = theParent;
		midibus = new MidiBus(app, midiInputIndex, midiOutputIndex);
	
		//reset();  // No need to reset the Vixid. Keep it's current state
		
		boolean[] trackVideoIn = new boolean[4];
		
		// create the 4 tracks
		for(int i=0; i < track.length; i++){
			
			for(int j=0; j < trackVideoIn.length; j++){
				trackVideoIn[j] = videoIn[(i*4)+j];
			}
			track[i] = new Track(this, i, trackVideoIn);
		}
		
	}

	/**
	 * Set the Vixid to be in Compositing or Battle mode.
	 * @param mode
	 * 		0 : Compositing mode, 1 : Battle 2*2 mode 
	 */
	public void setMixMode(int mode){
		if( mode == 0 ){
			// Compositing mode
			mixMode = 0;
			sendMidi(0, 0);
		}
		if( mode == 1 ){
			// Battle 2*2 mode
			mixMode = 1;
			sendMidi(0, 120);
		}
	}
	
	/**
	 * Set the configuration for the selected output
	 * @param outputId
	 * 		0 : output1, 1 : output2, 2 : output3
	 * @param config
	 * 		0 : master, 1 : master2, 2 : preview, 3 : auto preview
	 */
	public void setOutputConfig(int outputId, int config){
		if((outputId >=0 ) && (outputId <=2) && (config >= 0 ) && (config <=3 )){
			output[outputId] = config;
			sendMidi((outputId*2)+1, config*32);
		}
	}
	
	/**
	 * Set the preview point for the selected preview
	 * @param previewId
	 * 		0 : preview1, 1 : preview2, 2 : preview3
	 * @param point
	 * 		0 : Track1 PreFx, 1 : Track1 PostFx, 2 : Track1 Keyer, 3 : Track1 Out Blend, 4 : Track2 PreFx, 5 : Track2 PostFx, 6 : Track2 Keyer, 7 : Track2 Out Blend, 8 : Track3 PreFx, 9 : Track3 PostFx, 10 : Track3 Keyer, 11 : Track3 Out Blend, 12 : Track4 PreFx, 13 : Track4 PostFx, 14 : Track4 Keyer, 15 : Track4 Out Blend 
	 */
	public void setPreview(int previewId, int point){
		if((previewId >=0 ) && (previewId <=2) && (point >= 0 ) && (point <=15 )){
			preview[previewId] = point;
			sendMidi((previewId*2)+2, preview[previewId]*8);
		}
	}
	
	/**
	 * Set the preview point for the selected preview
	 * @param previewId
	 * 		0 : preview1, 1 : preview2, 2 : preview3
	 * @param trackId
	 * 		0 : Track1, 1 : Track2, 2 : Track3, 3 : Track4
	 * @param point
	 * 		0 : PreFx, 1 : PostFx, 2 : Keyer, 3 : Out Blend
	 */
	public void setPreview(int previewId, int trackId, int point){
		if((previewId >=0 ) && (previewId <=2) && (trackId >= 0 ) && (trackId <=3 ) && (point >= 0 ) && (point <=15 )){
			preview[previewId] = (trackId*4)+point;
			sendMidi((previewId*2)+2, preview[previewId]*8);
		}
	}

	/*
	public void reset(){
		setMixMode(0); // Compositing mode
		setOutputConfig(0, 0); // Output1 Master
		setOutputConfig(1, 0); // Output2 Master
		setOutputConfig(2, 2); // Output3 Preview
		setPreview(0, 0, 3); // Track1 out blend
		setPreview(1, 1, 3); // Track2 out blend
		setPreview(2, 2, 3); // Track3 out blend
		setPreview(3, 3, 3); // Track4 out blend
		PApplet.println("reset");
	}
	*/
	
	/**
	 * Returns the list of available midi devices. It's just a call to theMidibus library function that does this.
	 */
	public static void midiList() {
		MidiBus.list();
	}

	private void sendMidi(int channel, int value) {
		midibus.sendControllerChange(4, channel, value);
	}

	private void sendMidi(int channel, float value) {
		midibus.sendControllerChange(4, channel, (int)(value));
	}
	
	private void about() {
		System.out.println("Vixid 0.1.2 by Julien Deswaef (aka. Juego) http://xuv.be");
	}

	/**
	 * return the version of the library.
	 * 
	 * @return String
	 */
	public static String version() {
		return VERSION;
	}

}

